package org.iwsec.pws.pwscup2016;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.function.Function;

public class ExampleUtility {
	protected ExampleUtility() {

	}

	static final public int INDEX_M_CUSTOMER = 0;
	static final public int INDEX_M_GENDER = 1;
	static final public int INDEX_M_DOB = 2;
	static final public int INDEX_M_COUNTRY = 3;
	static final public int COLUMN_COUNT_M = 4;

	static final public int INDEX_T_CUSTOMER = 0;
	static final public int INDEX_T_INVOICE = 1;
	static final public int INDEX_T_DATE_YMD = 2;
	static final public int INDEX_T_DATE_HM = 3;
	static final public int INDEX_T_STOCK = 4;
	static final public int INDEX_T_PRICE = 5;
	static final public int INDEX_T_QUANTITY = 6;
	static final public int COLUMN_COUNT_T = 7;

	static public <T> List<? extends T> getList(final List<? extends List<? extends T>> list, int index) {
		return new AbstractList<T>() {

			@Override
			public T get(int i) {
				return list.get(i).get(index);
			}

			@Override
			public int size() {
				return list.size();
			}
		};
	}

	/**
	 * @param option null ȂW͂ǂݍށiĂ܂̂łP񂫂jB
	 */
	static public ArrayList<ArrayList<ArrayList<String>>> read(Path option) throws IOException {
		if (option != null) {
			try (BufferedReader reader = Files.newBufferedReader(option, StandardCharsets.ISO_8859_1)) {
				return read(reader);
			}
		}
		try (BufferedReader reader = new BufferedReader(new InputStreamReader(System.in, StandardCharsets.ISO_8859_1))) {
			return read(reader);
		}
	}

	static public ArrayList<ArrayList<ArrayList<String>>> read(BufferedReader reader) throws IOException {
		ArrayList<ArrayList<ArrayList<String>>> tableList = new ArrayList<>();
		String line;
		while ((line = reader.readLine()) != null) {
			ArrayList<ArrayList<String>> table = new ArrayList<>();
			int n = Integer.parseInt(line);
			for (int i = 0; i < n; i++) {
				line = reader.readLine();
				table.add(Utility.separate(line, ","));
			}
			tableList.add(table);
		}
		return tableList;
	}

	/**
	 * @param option null ȂWo͂ɏoiĂ܂̂łP񂫂jB
	 */
	static public void write(Path option, Collection<? extends List<? extends List<? extends String>>> tableCollection) throws IOException {
		if (option != null) {
			Files.createDirectories(option.getParent());
			try (BufferedWriter writer = Files.newBufferedWriter(option, StandardCharsets.ISO_8859_1)) {
				write(writer, tableCollection);
			}
			return;
		}
		try (BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(System.out, StandardCharsets.ISO_8859_1))) {
			write(writer, tableCollection);
		}
	}

	static public void writeWithoutCount(BufferedWriter writer, List<? extends List<? extends String>> table) throws IOException {
		for (List<? extends String> row : table) {
			writer.write(Utility.concat(row, ",").toString());
			writer.newLine();
		}
	}

	static public void write(BufferedWriter writer, Collection<? extends List<? extends List<? extends String>>> tableCollection) throws IOException {
		for (List<? extends List<? extends String>> table : tableCollection) {
			writer.write(String.valueOf(table.size()));
			writer.newLine();
			writeWithoutCount(writer, table);
		}
	}

	static public long parseDate(String ymd, String hm) {
		DateFormat dateFormat = new SimpleDateFormat("yyyy/M/d H:mm");
		dateFormat.setLenient(false);
		try {
			return dateFormat.parse(ymd + " " + hm).getTime();
		}
		catch (ParseException e) {
			throw new IllegalArgumentException(e);
		}
	}

	static public int parsePriceInPenny(String price) {
		int index = price.indexOf('.');
		if (index < 0) {
			price = price + ".00";
		}
		else if (index == price.length() - 2) {
			price = price + "0";
		}
		return Integer.parseInt(price.replaceFirst("\\.", ""));
	}

	static public ArrayList<ArrayList<String>> asTable(List<? extends Integer> indexList) {
		return asTable(indexList, i -> String.valueOf(i + 1));
	}

	static public <T> ArrayList<ArrayList<String>> asTable(List<? extends T> list, Function<? super T, ? extends String> f) {
		ArrayList<ArrayList<String>> table = new ArrayList<>();
		for (T element : list) {
			table.add(new ArrayList<>(Arrays.asList(f.apply(element))));
		}
		return table;
	}

	static public ArrayList<ArrayList<String>> copyTable(List<? extends List<? extends String>> table) {
		ArrayList<ArrayList<String>> copy = new ArrayList<>();
		for (List<? extends String> list : table) {
			copy.add(new ArrayList<>(list));
		}
		return copy;
	}

	/**
	 * @return c ȂnullAłȂ΃_ɑI񂾗vfB
	 */
	static public <T> T getRandomly(Random random, Collection<? extends T> c) {
		if (c.isEmpty()) {
			return null;
		}
		int i = random.nextInt(c.size());
		Iterator<? extends T> iterator = c.iterator();
		while (i-- > 0) {
			iterator.next();
		}
		return iterator.next();
	}

	static public <V> List<V> getColumnList(List<? extends List<V>> table, int column) {
		return new AbstractList<V>() {

			@Override
			public V get(int index) {
				return table.get(index).get(column);
			}

			@Override
			public int size() {
				return table.size();
			}

			@Override
			public V set(int index, V element) {
				return table.get(index).set(column, element);
			}
		};
	}

	static public ArrayList<Integer> shuffle(Random random, ArrayList<ArrayList<String>> mTable, ArrayList<ArrayList<String>> tTable) {
		ArrayList<Integer> indexList = new ArrayList<>();

		for (int r = 0; r < mTable.size(); r++) {
			mTable.get(r).add(String.valueOf(r));
		}
		Collections.shuffle(mTable, random);
		for (int r = 0; r < mTable.size(); r++) {
			ArrayList<String> list = mTable.get(r);
			indexList.add(Integer.parseInt(list.remove(list.size() - 1)));
		}

		Collections.shuffle(tTable, random);

		return indexList;
	}
}
