package org.iwsec.pws.pwscup2016;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.function.Function;

public class Utility {
	protected Utility() {

	}

	static final public int INDEX_M_CUSTOMER = 0;
	static final public int INDEX_M_GENDER = 1;
	static final public int INDEX_M_DOB = 2;
	static final public int INDEX_M_COUNTRY = 3;

	static final public int INDEX_T_CUSTOMER = 0;
	static final public int INDEX_T_INVOICE = 1;
	static final public int INDEX_T_DATE = 2;
	static final public int INDEX_T_STOCK = 3;
	static final public int INDEX_T_PRICE = 4;
	static final public int INDEX_T_QUANTITY = 5;

	/**
	 * c ̗vf separator pĘAԂB
	 */
	static public StringBuilder concat(Collection<?> c, String separator) {
		StringBuilder builder = new StringBuilder();
		for (Object o : c) {
			String s = o.toString();
			builder.append(separator);
			builder.append(s);
		}
		if (builder.length() > 0) {
			builder.delete(0, separator.length());
		}
		return builder;
	}

	/**
	 * String.indexOf(separator) g s ؂ʂ String  ArrayList ɂĕԂB
	 * s 󕶎񂾂Ƃ̂݋ƂȂB
	 * null ͕ԂȂB
	 */
	static public ArrayList<String> separate(String s, String separator) {
		ArrayList<String> stringList = new ArrayList<String>();
		if (s.isEmpty()) {
			return stringList;
		}
		int separatorLength = separator.length();
		int lastIndex = 0;
		int index;
		while ((index = s.indexOf(separator, lastIndex)) >= 0) {
			stringList.add(s.substring(lastIndex, index));
			lastIndex = index + separatorLength;
		}
		stringList.add(s.substring(lastIndex));
		return stringList;
	}

	static public <T> List<? extends T> getList(final List<? extends List<? extends T>> list, int index) {
		return new AbstractList<T>() {

			@Override
			public T get(int i) {
				return list.get(i).get(index);
			}

			@Override
			public int size() {
				return list.size();
			}
		};
	}

	/**
	 * @param option null ȂW͂ǂݍށiĂ܂̂łP񂫂jB
	 */
	static public ArrayList<ArrayList<ArrayList<String>>> read(Path option) throws IOException {
		if (option != null) {
			try (BufferedReader reader = Files.newBufferedReader(option, StandardCharsets.ISO_8859_1)) {
				return read(reader);
			}
		}
		try (BufferedReader reader = new BufferedReader(new InputStreamReader(System.in, StandardCharsets.ISO_8859_1))) {
			return read(reader);
		}
	}

	static public ArrayList<ArrayList<ArrayList<String>>> read(BufferedReader reader) throws IOException {
		ArrayList<ArrayList<ArrayList<String>>> tableList = new ArrayList<>();
		String line;
		while ((line = reader.readLine()) != null) {
			ArrayList<ArrayList<String>> table = new ArrayList<>();
			int n = Integer.parseInt(line);
			for (int i = 0; i < n; i++) {
				line = reader.readLine();
				table.add(separate(line, ","));
			}
			tableList.add(table);
		}
		return tableList;
	}

	/**
	 * @param option null ȂWo͂ɏoiĂ܂̂łP񂫂jB
	 */
	static public void write(Path option, Collection<? extends List<? extends List<? extends String>>> tableCollection) throws IOException {
		if (option != null) {
			Files.createDirectories(option.getParent());
			try (BufferedWriter writer = Files.newBufferedWriter(option, StandardCharsets.ISO_8859_1)) {
				write(writer, tableCollection);
			}
			return;
		}
		try (BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(System.out, StandardCharsets.ISO_8859_1))) {
			write(writer, tableCollection);
		}
	}

	static public void write(BufferedWriter writer, Collection<? extends List<? extends List<? extends String>>> tableCollection) throws IOException {
		for (List<? extends List<? extends String>> table : tableCollection) {
			writer.write(String.valueOf(table.size()));
			writer.newLine();
			for (List<? extends String> row : table) {
				writer.write(concat(row, ",").toString());
				writer.newLine();
			}
		}
	}

	static public long parseDate(String date) {
		try {
			return new SimpleDateFormat("yyyy/M/d H:mm").parse(date).getTime();
		}
		catch (ParseException e) {
			throw new IllegalArgumentException(date);
		}
	}

	static public int parsePriceInPenny(String price) {
		int index = price.indexOf('.');
		if (index < 0) {
			price = price + ".00";
		}
		else if (index == price.length() - 2) {
			price = price + "0";
		}
		return Integer.parseInt(price.replaceFirst("\\.", ""));
	}

	static public ArrayList<ArrayList<String>> asTable(List<? extends Integer> indexList) {
		return asTable(indexList, i -> String.valueOf(i + 1));
	}

	static public <T> ArrayList<ArrayList<String>> asTable(List<? extends T> list, Function<? super T, ? extends String> f) {
		ArrayList<ArrayList<String>> table = new ArrayList<>();
		for (T element : list) {
			table.add(new ArrayList<>(Arrays.asList(f.apply(element))));
		}
		return table;
	}

	static public ArrayList<ArrayList<String>> copyTable(List<? extends List<? extends String>> table) {
		ArrayList<ArrayList<String>> copy = new ArrayList<>();
		for (List<? extends String> list : table) {
			copy.add(new ArrayList<>(list));
		}
		return copy;
	}

	/**
	 * @return c ȂnullAłȂ΃_ɑI񂾗vfB
	 */
	static public <T> T getRandomly(Random random, Collection<? extends T> c) {
		if (c.isEmpty()) {
			return null;
		}
		int i = random.nextInt(c.size());
		Iterator<? extends T> iterator = c.iterator();
		while (i-- > 0) {
			iterator.next();
		}
		return iterator.next();
	}
}
