package org.iwsec.pws.pwscup2016.ano;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.function.Function;

import org.iwsec.pws.pwscup2016.ExampleUtility;

/**
 * ڋqIDƓ`[ID̓VbtAN̓͋ψꉻAiID͋ψꉻAPƐʂ͗L1A}X^[͋ψꉻB
 * @author yamaoka.yuji@jp.fujitsu.com
 *
 */
public class Ba {
	static public int round(int value) {
		if (value <= 0) {
			throw new IllegalArgumentException(String.format("value must be positive: %s", value));
		}
		String s = String.valueOf(value);
		int v = Integer.parseInt(s.substring(0, 1));
		int d = s.length() - 1;
		while (d-- > 0) {
			v *= 10;
		}
		return v;
	}

	static protected void change(ArrayList<ArrayList<String>> table, Function<? super String, ? extends String>... functions) {
		table.forEach(list -> {
			for (int i = 0; i < list.size(); i++) {
				list.set(i, functions[i].apply(list.get(i)));
			}
		});
	}

	protected void change(Random random, ArrayList<ArrayList<String>> mTable, ArrayList<ArrayList<String>> tTable) {
		Map<? extends String, ? extends String> customerMap = newShuffledMap(random, tTable, ExampleUtility.INDEX_T_CUSTOMER);
		Map<? extends String, ? extends String> invoiceMap = newShuffledMap(random, tTable, ExampleUtility.INDEX_T_INVOICE);

		changeMaster(customerMap, mTable);
		changeTransaction(customerMap, invoiceMap, tTable);
	}

	protected void changeTransaction(Map<? extends String, ? extends String> customerMap, Map<? extends String, ? extends String> invoiceMap, ArrayList<ArrayList<String>> table) {
		Function<String, String>[] functions = new Function[ExampleUtility.COLUMN_COUNT_T];
		functions[ExampleUtility.INDEX_T_CUSTOMER] = s -> customerMap.get(s);
		functions[ExampleUtility.INDEX_T_INVOICE] = s -> invoiceMap.get(s);
		functions[ExampleUtility.INDEX_T_DATE_YMD] = s -> s.substring(0, s.lastIndexOf("/")) + "/1";
		functions[ExampleUtility.INDEX_T_DATE_HM] = s -> "0:00";
		functions[ExampleUtility.INDEX_T_STOCK] = s -> "M";
		functions[ExampleUtility.INDEX_T_PRICE] = s -> String.valueOf(round((ExampleUtility.parsePriceInPenny(s) + 100 - 1) / 100));
		functions[ExampleUtility.INDEX_T_QUANTITY] = s -> String.valueOf(round(Integer.parseInt(s)));
		change(table, functions);
	}

	protected void changeMaster(Map<? extends String, ? extends String> customerMap, ArrayList<ArrayList<String>> table) {
		Function<String, String>[] functions = new Function[ExampleUtility.COLUMN_COUNT_M];
		functions[ExampleUtility.INDEX_M_CUSTOMER] = s -> customerMap.get(s);
		functions[ExampleUtility.INDEX_M_GENDER] = s -> "f";
		functions[ExampleUtility.INDEX_M_DOB] = s -> "1960/1/1";
		functions[ExampleUtility.INDEX_M_COUNTRY] = s -> "United Kingdom";
		change(table, functions);
	}

	protected Map<? extends String, ? extends String> newShuffledMap(Random random, List<? extends List<? extends String>> table, int column) {
		ArrayList<? extends String> list = new ArrayList<>(new LinkedHashSet<>(ExampleUtility.getColumnList(table, column)));
		ArrayList<? extends String> newList = new ArrayList<>(list);
		Collections.shuffle(newList, random);

		Map<String, String> map = new HashMap<>();
		for (int i = 0; i < list.size(); i++) {
			map.put(list.get(i), newList.get(i));
		}
		return map;
	}

	static public void main(String... args) throws IOException {
		Random random = new Random();

		Path in = null;
		Path out = null;
		if (args.length > 0) {
			in = Paths.get(args[0]);
			if (args.length > 1) {
				out = Paths.get(args[1]);
			}
		}

		ArrayList<ArrayList<ArrayList<String>>> tableList = ExampleUtility.read(in);

		Ba one = new Ba();
		tableList.subList(2, tableList.size()).clear();
		one.change(random, tableList.get(0), tableList.get(1));
		List<? extends Integer> indexList = ExampleUtility.shuffle(random, tableList.get(0), tableList.get(1));
		tableList.add(ExampleUtility.asTable(indexList));

		ExampleUtility.write(out, tableList);
	}
}
