package org.iwsec.pws.pwscup2016.check;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import org.iwsec.pws.pwscup2016.Utility;

public class CheckUtility {
	protected CheckUtility() {
	}

	static public ArrayList<Pattern> readPatterns(String name) throws IOException {
		ArrayList<Pattern> patterns = new ArrayList<>();
		try (InputStream inputStream = CheckUtility.class.getResourceAsStream(name)) {
			try (BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream, StandardCharsets.UTF_8))) {
				String line;
				while ((line = reader.readLine()) != null) {
					try {
						List<String> list = Utility.separate(line, "\t");
						int n = Integer.parseInt(list.get(0));
						if (n < 0) {
							throw new IllegalArgumentException(String.format("head number must be non-negative: %s", n));
						}
						StringBuilder builder = new StringBuilder(list.get(1));
						while (n-- > 0) {
							line = reader.readLine();
							if (line == null) {
								throw new IllegalArgumentException("not enough lines");
							}
							builder.append(System.getProperty("line.separator")).append(line);
						}
						Pattern pattern = Pattern.compile(builder.toString());
						patterns.add(pattern);
					}
					catch (RuntimeException e) {
						throw new IllegalArgumentException(String.format("Invalid format: %s", name), e);
					}
				}
			}
		}
		return patterns;
	}

}
