package org.iwsec.pws.pwscup2016.check;

import java.io.BufferedReader;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.function.Predicate;
import java.util.regex.Pattern;

import org.iwsec.pws.pwscup2016.Utility;

public class CheckerM {
	static final public int MAX_COLUMN_COUNT = 48;

	static final public List<? extends Pattern> PATTERNS;

	static {
		try {
			PATTERNS = Collections.unmodifiableList(CheckUtility.readPatterns("patternsM.txt"));
		}
		catch (IOException e) {
			throw new ExceptionInInitializerError(e);
		}
	}

	static public void main(String... args) throws IOException {
		Collection<? extends Message> messages = run(newColumnPredicates(), args);
		if (messages.isEmpty()) {
			return;
		}
		for (Message m : messages) {
			System.err.println(m);
		}
		System.exit(1);
	}

	static public boolean canParse(String ymd) {
		try {
			Utility.parseDate(ymd);
			return true;
		}
		catch (IllegalArgumentException e) {
			return false;
		}
	}

	static public ArrayList<Predicate<String>> newColumnPredicates() {
		ArrayList<Predicate<String>> predicates = new ArrayList<>(PATTERNS.size());
		for (Pattern p : PATTERNS) {
			predicates.add(s -> p.matcher(s).matches());
		}

		int index = 2;
		Predicate<String> predicate = predicates.get(index);
		Predicate<String> newPredicate = (s -> predicate.test(s) && canParse(s));
		predicates.set(index, newPredicate);
		return predicates;
	}

	static public Collection<? extends Message> run(Collection<? extends Predicate<? super String>> columnPredicates, String... args) throws IOException {
		if (args.length < 1) {
			throw new IllegalArgumentException(String.format("A master data file path must be specified as the argument."));
		}
		Path path = Paths.get(args[0]);

		if (args.length < 2) {
			throw new IllegalArgumentException(String.format("The max row count of the master data must be specified as the second argument."));
		}
		int maxRowCount = Integer.parseInt(args[1]);

		return newMessages(maxRowCount, path);
	}

	static public Collection<? extends Message> newMessages(int maxRowCount, Path path) throws IOException {
		try (BufferedReader reader = Files.newBufferedReader(path, StandardCharsets.ISO_8859_1)) {
			return newMessages(maxRowCount, reader);
		}
	}

	static public Collection<? extends Message> newMessages(int maxRowCount, BufferedReader reader) throws IOException {
		ReaderWithCheck r = new ReaderWithCheck(reader, MAX_COLUMN_COUNT, maxRowCount);
		return new CheckerT().newMessages(newColumnPredicates(), CheckerT.newRowCheck(), r);
	}
}
