package org.iwsec.pws.pwscup2016.check;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class CheckerMT {
	static public void main(String... args) throws IOException {
		if (args.length <= 0) {
			throw new IllegalArgumentException(String.format("A master data file path must be specified as the 1st argument."));
		}
		Path mPath = Paths.get(args[0]);

		if (args.length <= 1) {
			throw new IllegalArgumentException(String.format("A transaction data file path must be specified as the 2nd argument."));
		}
		Path tPath = Paths.get(args[1]);

		if (args.length <= 2) {
			throw new IllegalArgumentException(String.format("The max row count of the master data must be specified as the 3rd argument."));
		}
		int maxRowCount = Integer.parseInt(args[2]);

		CheckerM.main(new String[] { mPath.toString(), String.valueOf(maxRowCount) });
		CheckerT.main(new String[] { tPath.toString() });

		Collection<? extends Message> messages = newMessages(mPath, tPath);
		if (messages.isEmpty()) {
			return;
		}
		for (Message m : messages) {
			System.err.println(m);
		}
		System.exit(1);
	}

	static public HashSet<String> readIdSet(Path mPath) throws IOException {
		try (BufferedReader reader = Files.newBufferedReader(mPath, StandardCharsets.ISO_8859_1)) {
			return readIdSet(reader);
		}
	}

	static public HashSet<String> readIdSet(BufferedReader reader) throws IOException {
		HashSet<String> idSet = new HashSet<>();

		String line;
		while ((line = reader.readLine()) != null) {
			String id = line.substring(0, line.indexOf(","));
			idSet.add(id);
		}
		return idSet;
	}

	static public Collection<? extends Message> newMessages(Path mPath, Path tPath) throws IOException {
		Set<? extends String> idSet = readIdSet(mPath);
		return newMessages(idSet, tPath);
	}

	static public Collection<? extends Message> newMessages(Set<? extends String> idSet, Path tPath) throws IOException {
		try (BufferedReader reader = Files.newBufferedReader(tPath, StandardCharsets.ISO_8859_1)) {
			return new CheckerMT().newMessages(idSet, reader);
		}
	}

	public Collection<? extends Message> newMessages(Set<? extends String> idSet, BufferedReader reader) throws IOException {
		LineNumberReader lineNumberReader;
		if (reader instanceof LineNumberReader) {
			lineNumberReader = (LineNumberReader) reader;
		}
		else {
			lineNumberReader = new LineNumberReader(reader);
		}

		List<Message> messages = new ArrayList<>();
		String line;
		while ((line = lineNumberReader.readLine()) != null) {
			String id = line.substring(0, line.indexOf(","));
			if (!idSet.contains(id)) {
				String message = String.format("the value of CustomerID in the transaction data must be included in the corresponding master data (%s)", id);
				messages.add(new Message(lineNumberReader.getLineNumber(), message));
			}
		}
		return messages;
	}
}
