package org.iwsec.pws.pwscup2016.check;

import java.io.IOException;
import java.io.Reader;
import java.util.Objects;

public class ReaderWithCheck {
	private final int maxColumnCount;
	private final int maxRowCount;
	private final Reader reader;

	private int lineNumber;

	public ReaderWithCheck(Reader reader, int maxColumnCount, int maxRowCount) {
		this.reader = Objects.requireNonNull(reader);
		this.maxColumnCount = maxColumnCount;
		this.maxRowCount = maxRowCount;

		if (maxColumnCount <= 0) {
			throw new IllegalArgumentException(String.format("maxColumnCount must be positive: %s", maxColumnCount));
		}
		if (maxRowCount <= 0) {
			throw new IllegalArgumentException(String.format("maxRowCount must be positive: %s", maxRowCount));
		}
	}

	protected int getMaxColumnCount() {
		return maxColumnCount;
	}

	protected int getMaxRowCount() {
		return maxRowCount;
	}

	public int getLineNumber() {
		return lineNumber;
	}

	public String readLine() throws IOException, CheckException {
		int c = reader.read();
		if (c < 0) {
			return null;
		}
		lineNumber++;
		if (lineNumber > getMaxRowCount()) {
			throw new CheckException(String.format("line count must be at most %d", getMaxRowCount()));
		}
		StringBuilder builder = new StringBuilder();
		do {
			switch (c) {
			case '\r':
				c = reader.read();
				if (c != '\n') {
					throw new CheckException("newline must be LF (\\n) or CRLF (\\r\\n)");
				}
				// FALLTHROUGH
			case -1:
				// FALLTHROUGH
			case '\n':
				return builder.toString();
			default:
				builder.append((char) c);
				if (builder.length() > getMaxColumnCount()) {
					throw new CheckException(String.format("line length must be at most %d", getMaxColumnCount()));
				}
				break;
			}
			c = reader.read();
		}
		while (true);
	}

}
