package org.iwsec.pws.pwscup2016.re;

import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

import org.iwsec.pws.pwscup2016.Utility;

public class ReT {
	static protected class StockSummary {
		static public StockSummary newSummary(String date, String price, String quantity) {
			long dateInMinute = Utility.parseDate(date) / (1000 * 60);
			int penny = Utility.parsePriceInPenny(price);
			long subtotal = (long) penny * Integer.parseInt(quantity);
			return new StockSummary(dateInMinute, subtotal);
		}

		private final long dateInMinute;
		private final long subtotal;

		public StockSummary(long dateInMinute, long subtotal) {
			this.dateInMinute = dateInMinute;
			this.subtotal = subtotal;

			if (subtotal <= 0) {
				throw new IllegalArgumentException(String.format("subtotal must be positive: %s", subtotal));
			}
		}

		public long getDateInMinute() {
			return dateInMinute;
		}

		public long getSubtotal() {
			return subtotal;
		}

		@Override
		public String toString() {
			return String.format("dateInMinute=%s, subtotal=%s", dateInMinute, subtotal);
		}
	}

	static protected long getSubtotal(List<? extends StockSummary> summaries) {
		return summaries.stream().mapToLong(ss -> ss.getSubtotal()).sum();
	}

	static protected double getDateAverage(List<? extends StockSummary> summaries) {
		double sum = summaries.stream().mapToDouble(ss -> ss.getDateInMinute()).sum();
		return sum / summaries.size();
	}

	protected Map<? extends String, ? extends Map<? extends String, ? extends List<? extends StockSummary>>> newCustomerStockSummaries(List<? extends List<? extends String>> mTable, List<? extends List<? extends String>> tTable) {
		Map<String, ? extends Map<? extends String, ? extends List<? extends StockSummary>>> customerStockSummaries = newCustomerStockSummaries(tTable);
		Collection<String> customers = new HashSet<>(Utility.getList(mTable, Utility.INDEX_M_CUSTOMER));
		customerStockSummaries.keySet().retainAll(customers);
		return customerStockSummaries;
	}

	protected Map<String, ? extends Map<? extends String, ? extends List<? extends StockSummary>>> newCustomerStockSummaries(List<? extends List<? extends String>> tTable) {
		Map<String, HashMap<String, ArrayList<StockSummary>>> customerStockSummaries = new HashMap<>();
		for (List<? extends String> row : tTable) {
			String customer = row.get(Utility.INDEX_T_CUSTOMER);
			HashMap<String, ArrayList<StockSummary>> stockSummaries = customerStockSummaries.get(customer);
			if (stockSummaries == null) {
				customerStockSummaries.put(customer, stockSummaries = new HashMap<>());
			}
			String stock = row.get(Utility.INDEX_T_STOCK);
			ArrayList<StockSummary> summaries = stockSummaries.get(stock);
			if (summaries == null) {
				stockSummaries.put(stock, summaries = new ArrayList<>());
			}
			String date = row.get(Utility.INDEX_T_DATE);
			String price = row.get(Utility.INDEX_T_PRICE);
			String quantity = row.get(Utility.INDEX_T_QUANTITY);
			summaries.add(StockSummary.newSummary(date, price, quantity));
		}
		return customerStockSummaries;
	}

	protected List<? extends Integer> newIndexList(Random random, List<? extends List<? extends String>> mTable, List<? extends List<? extends String>> tTable, List<? extends List<? extends String>> m2Table, List<? extends List<? extends String>> t2Table) {
		List<? extends String> customerList = Utility.getList(mTable, Utility.INDEX_M_CUSTOMER);
		List<? extends String> customerList2 = Utility.getList(m2Table, Utility.INDEX_M_CUSTOMER);

		// gUNV CustomerID ł܂Ƃ߂
		Map<? extends String, ? extends Map<? extends String, ? extends List<? extends StockSummary>>> customerStockSummaries = newCustomerStockSummaries(mTable, tTable);
		Map<? extends String, ? extends Map<? extends String, ? extends List<? extends StockSummary>>> customerStockSummaries2 = newCustomerStockSummaries(m2Table, t2Table);

		Set<String> noTransactionCustomerSet = new HashSet<>(customerList);
		noTransactionCustomerSet.removeAll(customerStockSummaries.keySet());

		List<Integer> indexList = new ArrayList<>();
		for (String customer2 : customerList2) {
			String customer = getCustomer(random, customerStockSummaries, noTransactionCustomerSet, customerStockSummaries2.get(customer2));
			int index;
			if (customer == null) { // customer̐ɎsAƓso͂
				index = indexList.size();
			}
			else {
				index = customerList.indexOf(customer);
				if (index < 0) {
					throw new AssertionError(customer);
				}
			}
			indexList.add(index);
		}
		return indexList;
	}

	protected String getCustomer(Random random, Map<? extends String, ? extends Map<? extends String, ? extends List<? extends StockSummary>>> customerStockSummaries, Collection<? extends String> noTransactionCustomerCollection, Map<? extends String, ? extends List<? extends StockSummary>> stockSummaries2) {
		if (stockSummaries2 == null) {
			return Utility.getRandomly(random, noTransactionCustomerCollection);
		}

		// iWł߂customerӂȂAŌ߂B
		SortedMap<? extends Double, ? extends Map<? extends String, ? extends Set<? extends String>>> closenessCustomerStock = newClosenessCustomerStock(customerStockSummaries, stockSummaries2.keySet());
		if (closenessCustomerStock.isEmpty()) {
			return Utility.getRandomly(random, noTransactionCustomerCollection);
		}
		Map<? extends String, ? extends Set<? extends String>> customerStock = closenessCustomerStock.get(closenessCustomerStock.lastKey());
		if (customerStock.size() == 1) {
			return customerStock.keySet().iterator().next();
		}

		// iW̏vXgł߂customerӂȂAŌ߂B
		SortedMap<? extends Long, ? extends Set<? extends String>> distanceCustomer = newSubtotalDistanceCustomer(stockSummaries2, customerStockSummaries, customerStock);
		Set<? extends String> customers = distanceCustomer.get(distanceCustomer.firstKey());
		if (customers.size() == 1) {
			return customers.iterator().next();
		}

		// iW̓Xgł߂customer烉_Ɍ߂B
		SortedMap<? extends Double, ? extends Set<? extends String>> dateDistanceCustomer = newDateDistanceCustomer(stockSummaries2, customerStockSummaries, customerStock, customers);
		customers = dateDistanceCustomer.get(dateDistanceCustomer.firstKey());
		return Utility.getRandomly(random, customers);
	}

	private SortedMap<? extends Double, ? extends Map<? extends String, ? extends Set<? extends String>>> newClosenessCustomerStock(Map<? extends String, ? extends Map<? extends String, ? extends List<? extends StockSummary>>> customerStockSummaries, Set<? extends String> stock2Set) {
		SortedMap<Double, HashMap<String, HashSet<String>>> closenessCustomerStock = new TreeMap<>();
		customerStockSummaries.forEach((customer, stockSummaries) -> {
			Set<? extends String> stockSet = stockSummaries.keySet();
			HashSet<String> intersection = new HashSet<>(stockSet);
			intersection.retainAll(stock2Set);
			if (intersection.isEmpty()) {
				return;
			}
			Set<String> union = new HashSet<>(stockSet);
			union.addAll(stock2Set);
			double closeness = (double) intersection.size() / union.size();
			HashMap<String, HashSet<String>> cs = closenessCustomerStock.get(closeness);
			if (cs == null) {
				closenessCustomerStock.put(closeness, cs = new HashMap<>());
			}
			cs.put(customer, intersection);
		});
		return closenessCustomerStock;
	}

	private SortedMap<? extends Long, ? extends Set<? extends String>> newSubtotalDistanceCustomer(Map<? extends String, ? extends List<? extends StockSummary>> stockSummaries2, Map<? extends String, ? extends Map<? extends String, ? extends List<? extends StockSummary>>> customerStockSummaries, Map<? extends String, ? extends Set<? extends String>> customerStock) {
		SortedMap<Long, HashSet<String>> distanceCustomer = new TreeMap<>();
		customerStock.forEach((customer, stockSet) -> {
			long distance = 0;
			for (String stock : stockSet) {
				List<? extends StockSummary> summaries = customerStockSummaries.get(customer).get(stock);
				List<? extends StockSummary> summaries2 = stockSummaries2.get(stock);
				distance += Math.abs(getSubtotal(summaries) - getSubtotal(summaries2));
			}
			HashSet<String> cs = distanceCustomer.get(distance);
			if (cs == null) {
				distanceCustomer.put(distance, cs = new HashSet<>());
			}
			cs.add(customer);
		});
		return distanceCustomer;
	}

	private SortedMap<? extends Double, ? extends Set<? extends String>> newDateDistanceCustomer(Map<? extends String, ? extends List<? extends StockSummary>> stockSummaries2, Map<? extends String, ? extends Map<? extends String, ? extends List<? extends StockSummary>>> customerStockSummaries, Map<? extends String, ? extends Set<? extends String>> customerStock, Set<? extends String> customers) {
		SortedMap<Double, HashSet<String>> distanceCustomer = new TreeMap<>();
		for (String customer : customers) {
			double distance = 0;
			for (String stock : customerStock.get(customer)) {
				List<? extends StockSummary> summaries = customerStockSummaries.get(customer).get(stock);
				List<? extends StockSummary> summaries2 = stockSummaries2.get(stock);
				distance += Math.abs(getDateAverage(summaries) - getDateAverage(summaries2));
			}
			HashSet<String> cs = distanceCustomer.get(distance);
			if (cs == null) {
				distanceCustomer.put(distance, cs = new HashSet<>());
			}
			cs.add(customer);
		}
		return distanceCustomer;
	}

	static public void main(String... args) throws IOException {
		Path in = null;
		Path out = null;
		if (args.length > 0) {
			in = Paths.get(args[0]);
			if (args.length > 1) {
				out = Paths.get(args[1]);
			}
		}

		ArrayList<ArrayList<ArrayList<String>>> tableList = Utility.read(in);

		Random random = new Random(0);
		ReT one = new ReT();
		List<? extends Integer> indexList = one.newIndexList(random, tableList.get(0), tableList.get(1), tableList.get(2), tableList.get(3));
		tableList.clear();
		tableList.add(Utility.asTable(indexList));

		Utility.write(out, tableList);
	}
}
